@@ -2,7 +2,7 @@
 // server.cpp
 // ~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -11,7 +11,7 @@
 #include <ctime>
 #include <iostream>
 #include <string>
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/enable_shared_from_this.hpp>
 #include <asio.hpp>
@@ -31,9 +31,9 @@ class tcp_connection
 public:
   typedef boost::shared_ptr<tcp_connection> pointer;
 
-  static pointer create(asio::io_service& io_service)
+  static pointer create(asio::io_context& io_context)
   {
-    return pointer(new tcp_connection(io_service));
+    return pointer(new tcp_connection(io_context));
   }
 
   tcp::socket& socket()
@@ -52,8 +52,8 @@ public:
   }
 
 private:
-  tcp_connection(asio::io_service& io_service)
-    : socket_(io_service)
+  tcp_connection(asio::io_context& io_context)
+    : socket_(io_context)
   {
   }
 
@@ -69,8 +69,9 @@ private:
 class tcp_server
 {
 public:
-  tcp_server(asio::io_service& io_service)
-    : acceptor_(io_service, tcp::endpoint(tcp::v4(), 13))
+  tcp_server(asio::io_context& io_context)
+    : io_context_(io_context),
+      acceptor_(io_context, tcp::endpoint(tcp::v4(), 13))
   {
     start_accept();
   }
@@ -79,7 +80,7 @@ private:
   void start_accept()
   {
     tcp_connection::pointer new_connection =
-      tcp_connection::create(acceptor_.get_io_service());
+      tcp_connection::create(io_context_);
 
     acceptor_.async_accept(new_connection->socket(),
         boost::bind(&tcp_server::handle_accept, this, new_connection,
@@ -97,6 +98,7 @@ private:
     start_accept();
   }
 
+  asio::io_context& io_context_;
   tcp::acceptor acceptor_;
 };
 
@@ -104,9 +106,9 @@ int main()
 {
   try
   {
-    asio::io_service io_service;
-    tcp_server server(io_service);
-    io_service.run();
+    asio::io_context io_context;
+    tcp_server server(io_context);
+    io_context.run();
   }
   catch (std::exception& e)
   {
